// SCCSID "%Z% %W%  %I% %E% %U%"
/*
 * <N_OCO_COPYRIGHT>
 * Licensed Materials - Property of IBM
 * 
 * 5724-H72, 5655-R36, 5724-L26, 5655-L82     
 * 
 * (c) Copyright IBM Corp. 2008, 2009 All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corp.
 * <NOC_COPYRIGHT>
 */

package com.ibm.scenario.wmq.jms;

import java.util.Hashtable;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.directory.InitialDirContext;

/**
 * A simple application that sends a request message and then wait for, and
 * receive, the reply. Note: It is assumed that some other application will process the request
 * message and send the reply message.
 * 
 * Application makes use of fixed literals, any customisations will require re-compilation of this
 * source file. Application assumes that the named queue is empty prior to a run.
 * 
 * It is assumed that the initial context contains the named definitions for connection factory and
 * destination objects. To create such an initial context (on the file system), launch the JMSAdmin
 * tool and issue the following commands:
 * 
 * InitCtx> DEF CF(myCF)
 * 
 * InitCtx> DEF Q(myQueue) QUEUE(Q1)
 * 
 * The first command defines a connection factory object with the name 'myCF'. The second command
 * defines a destination object with the name 'myQueue' that maps to queue:///Q1.
 *
 *
 * Notes:
 * 
 * API type: JMS API (v1.1, unified domain)
 * 
 * Messaging domain: Point-to-point
 * 
 * Provider type: WebSphere MQ
 * 
 * Connection mode: Client connection
 * 
 * JNDI in use: Yes
 * 
 */
public class SimpleRequestor {

  // System exit status value (assume unset value to be 1)
  private static int status = 1;

  /**
   * Main method
   * 
   * @param args
   */
  public static void main(String[] args) {

    // Variables
    Connection connection = null;
    ConnectionFactory connectionFactory = null;
    Session session = null;
    Destination destination = null;
    Destination tempDestination = null;
    MessageProducer producer = null;
    MessageConsumer consumer = null;

    try {
      // Instantiate the initial context
      String contextFactory = "com.sun.jndi.fscontext.RefFSContextFactory";
      Hashtable environment = new Hashtable();
      environment.put(Context.INITIAL_CONTEXT_FACTORY, contextFactory);
      environment.put(Context.PROVIDER_URL, "file:/C:/JNDI-Directory");
      Context context = new InitialDirContext(environment);

      // Lookup the connection factory
      connectionFactory = (ConnectionFactory) context.lookup("myCF");
      System.out.println("> Connection factory located in JNDI.");

      // Lookup the destination
      destination = (Destination) context.lookup("myQueue");
      System.out.println("> Destination located in JNDI. ");

      // Create JMS objects
      System.out.println("> Creating connection to QueueManager.");
      connection = connectionFactory.createConnection();
      session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
      System.out.println("> Connection created.");

      // register an exception listener on the connection
      connection.setExceptionListener(new ExceptionListener() {

        public void onException(JMSException arg0) {
          System.out.println("Exception sent to exception listener");
          ExceptionHandling.handleException(arg0);
        }
      });

      // Create JMS producer & use temporary Queue for the responce message
      producer = session.createProducer(destination);
      tempDestination = session.createTemporaryQueue();

      // Create JMS consumer
      consumer = session.createConsumer(tempDestination);

      String text = "BakedBeans";
      System.out.println("\n> Sending stock request for '" + text + "'");
      TextMessage message = session
          .createTextMessage(text);

      // Set the JMSReplyTo
      message.setJMSReplyTo(tempDestination);

      // Start the connection
      connection.start();

      // And, send the request
      producer.send(message);
      System.out.println("> Sent Message ID=" + message.getJMSMessageID());

      // Now, receive the reply
      Message receivedMessage = consumer.receive(15000 * 1000); // in ms or 15 seconds
      if (receivedMessage != null) {
        System.out.println("\n> Received Message ID=" + receivedMessage.getJMSMessageID() + " for '" + ((TextMessage) receivedMessage).getText() + "'");
      }
      else {
        System.out.println("\n! No response message received in 15 seconds.");
      }

    }
    catch (NamingException ne) {
      System.out.println("The initial context could not be instantiated, or the lookup failed.");
      ExceptionHandling.handleException(ne);
    }
    catch (JMSException jmsex) {
      ExceptionHandling.handleException(jmsex);
    }
    finally {
      System.out.println("\n> Closing connection to QueueManager.");
      if (producer != null) {
        try {
          producer.close();
        }
        catch (JMSException jmsex) {
          System.out.println("Producer could not be closed.");
          ExceptionHandling.handleException(jmsex);
        }
      }
      if (consumer != null) {
        try {
          consumer.close();
        }
        catch (JMSException jmsex) {
          System.out.println("Consumer could not be closed.");
          ExceptionHandling.handleException(jmsex);
        }
      }

      if (session != null) {
        try {
          session.close();
        }
        catch (JMSException jmsex) {
          System.out.println("Session could not be closed.");
          ExceptionHandling.handleException(jmsex);
        }
      }

      if (connection != null) {
        try {
          connection.close();
        }
        catch (JMSException jmsex) {
          System.out.println("Connection could not be closed.");
          ExceptionHandling.handleException(jmsex);
        }
      }
      System.out.println("> Closed Connection.");
    }

    System.exit(status);
  } // end main()

}
